using System;
using System.Collections;
using DKMSCryptoCore;
using DKMSProtocol;
using DKMSErrorManager;

/**
* @file
* This file contains the implementation of the CryptoService Dispacher
* @ingroup CryptoService 
* @author Michele Castagnozzi
* @date 25/06/2005
* @version 1.0.0.3
*/
namespace CryptoService
{
	/**
	* This class manage the filtering and the Dispatching of the request wich come from by the DKMSClientInterface	 
	* @ingroup CSDispatcher 	  	 
	*/ 
	public class Dispatcher
	{
		#region Dispatcher class methods	

		/**
		 * Dispatcher class constructor
		 */ 
		public Dispatcher()
		{
			#if DEBUG               
                // local copy of current Containers instance
                _c = TestCryptoService._c;
				// local copy of current CryptoCore Instance
				_cc = TestCryptoService._cc;					
			#else                
                // local copy of current Containers instance
                _c = Service._c;
				// local copy of current CryptoCore Instance
				_cc = Service._cc;				
            #endif

                // Stmf Instance
			_stmf = new CryptoCore.Stmf(_cc);    

			// Smf Instance
			_smf = new CryptoCore.Smf(_cc);

			// ProtocolManager Instance
			_pm = new ProtocolManager(_cc.HostInfo.Info);
		}	

		/**
		* This is a function used in order to chose the requested crypto method
		* @param[in] RxProtocolBlock is the received DKMS Protocol data block
		* @param[out] TxProtocolBlock is the DKMS Protocol data block to be transmit
		*/ 
		public void CS_Dispacher(string RxProtocolBlock, out string TxProtocolBlock)
		{
			UInt32	ret = ErrorManager.DKMS_OK;
			
	
			// Initialization	
			TxProtocolBlock = string.Empty;				

			//////////////////////////////////////
			// Fase 1 Parse input data
			_pm.RxProtocolBlock = RxProtocolBlock;
			ret = _pm.XMLRXParser(out config);	            			
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);
				TxProtocolBlock = _pm.TxProtocolBlock;					
			}
			else
			{				
				// Verify Integrity					
				ret = _pm.VerifyDataIntegrity(config.security.hashValue);				
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);
					TxProtocolBlock = _pm.TxProtocolBlock;					
				}
				else
				{
                    // Select the policy container to be use
                    string delimStr = "|";
                    char[] delimiter = delimStr.ToCharArray();
                    string[] groups = config.header.userName.Split(delimiter);
                    string group = string.Empty;
                    Hashtable currentgroup = new Hashtable(20, 0.1f);

                    if (groups.Length != 2)
                    {
                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                        _pm.XMLTXParser_Set_Error(ret);
                        TxProtocolBlock = _pm.TxProtocolBlock;
                    }
                    else
                    {
                        if (groups[1]== "dkms_operators")
                            currentgroup = _c.operatorsgc.SyncOGContainer;
                        else if (groups[1] == "dkms_technicians")
                            currentgroup = _c.techniciansgc.SyncTGContainer;
                        else if (groups[1] == "dkms_swdevelopers")
                            currentgroup = _c.swdevelopersgc.SyncSGContainer;
                        else if (groups[1] == "dkms_inserters")
                            currentgroup = _c.insertersgc.SyncIGContainer;
                        else if (groups[1] == "dkms_custodians")
                            currentgroup = _c.custodiansgc.SyncCGContainer;
                        else if (groups[1] == "dkms_controllers")
                            currentgroup = _c.controllersgc.SyncCGContainer;
                        else if (groups[1] == "dkms_admins")
                            currentgroup = _c.adminsgc.SyncAGContainer;
                        else if (groups[1] == "dkms_swadmins")
                            currentgroup = _c.swadminsgc.SyncSGContainer;
                        else
                        {
                            ret = ErrorManager.DKMS_ACCESS_DENIED;
                            _pm.XMLTXParser_Set_Error(ret);
                            TxProtocolBlock = _pm.TxProtocolBlock;
                        }

                        if (ret == ErrorManager.DKMS_OK)
                        {
                            //////////////////////////////////////
                            // Fase 2 Call the right method
                            switch (config.body.cmdName)
                            {
                                case "DKMS_OpenSession":
                                    if (currentgroup.ContainsKey("OpenSession"))
                                    {
                                        CS_OpenSession();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CloseSession":
                                    if (currentgroup.ContainsKey("CloseSession"))
                                    {
                                        CS_CloseSession();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetSessionInfo":
                                    if (currentgroup.ContainsKey("GetSessionInfo"))
                                    {
                                        CS_GetSessionInfo();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_Login":
                                    if (currentgroup.ContainsKey("Login"))
                                    {
                                        CS_Login();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_Logout":
                                    if (currentgroup.ContainsKey("Logout"))
                                    {
                                        CS_Logout();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CreateObject":
                                    if (currentgroup.ContainsKey("CreateObject"))
                                    {
                                        CS_CreateObject();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_FindObjectsInit":
                                    if (currentgroup.ContainsKey("FindObjectsInit"))
                                    {
                                        CS_FindObjectsInit();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_FindObjects":
                                    if (currentgroup.ContainsKey("FindObjects"))
                                    {
                                        CS_FindObjects();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_FindObjectsFinal":
                                    if (currentgroup.ContainsKey("FindObjectsFinal"))
                                    {
                                        CS_FindObjectsFinal();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CompactFindObjects":
                                    if (currentgroup.ContainsKey("CompactFindObjects"))
                                    {
                                        CS_CompactFindObjects();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_EncryptInit":
                                    if (currentgroup.ContainsKey("EncryptInit"))
                                    {
                                        CS_EncryptInit();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_Encrypt":
                                    if (currentgroup.ContainsKey("Encrypt"))
                                    {
                                        CS_Encrypt();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CompactEncrypt":
                                    if (currentgroup.ContainsKey("CompactEncrypt"))
                                    {
                                        CS_CompactEncrypt();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_EncryptUpdate":
                                    if (currentgroup.ContainsKey("EncryptUpdate"))
                                    {
                                        CS_EncryptUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_EncryptFinal":
                                    if (currentgroup.ContainsKey("EncryptFinal"))
                                    {
                                        CS_EncryptFinal();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DecryptInit":
                                    if (currentgroup.ContainsKey("DecryptInit"))
                                    {
                                        CS_DecryptInit();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_Decrypt":
                                    if (currentgroup.ContainsKey("Decrypt"))
                                    {
                                        CS_Decrypt();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CompactDecrypt":
                                    if (currentgroup.ContainsKey("CompactDecrypt"))
                                    {
                                        CS_CompactDecrypt();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DecryptUpdate":
                                    if (currentgroup.ContainsKey("DecryptUpdate"))
                                    {
                                        CS_DecryptUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DecryptFinal":
                                    if (currentgroup.ContainsKey("DecryptFinal"))
                                    {
                                        CS_DecryptFinal();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetInfo":
                                    if (currentgroup.ContainsKey("GetInfo"))
                                    {
                                        CS_GetInfo();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetTokenInfo":
                                    if (currentgroup.ContainsKey("GetTokenInfo"))
                                    {
                                        CS_GetTokenInfo();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SeedRandom":
                                    if (currentgroup.ContainsKey("SeedRandom"))
                                    {
                                        CS_SeedRandom();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GenerateRandom":
                                    if (currentgroup.ContainsKey("GenerateRandom"))
                                    {
                                        CS_GenerateRandom();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CompactGenerateRandom":
                                    if (currentgroup.ContainsKey("CompactGenerateRandom"))
                                    {
                                        CS_CompactGenerateRandom();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DestroyObject":
                                    if (currentgroup.ContainsKey("DestroyObject"))
                                    {
                                        CS_DestroyObject();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetMechanismList":
                                    if (currentgroup.ContainsKey("GetMechanismList"))
                                    {
                                        CS_GetMechanismList();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetAttributeValue":
                                    if (currentgroup.ContainsKey("GetAttributeValue"))
                                    {
                                        CS_GetAttributeValue();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;    
                                    break;
                                case "DKMS_DigestInit":
                                    if (currentgroup.ContainsKey("DigestInit"))
                                    {
                                        CS_DigestInit();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_Digest":
                                    if (currentgroup.ContainsKey("Digest"))
                                    {
                                        CS_Digest();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CompactDigest":
                                    if (currentgroup.ContainsKey("CompactDigest"))
                                    {
                                        CS_CompactDigest();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DigestUpdate":
                                    if (currentgroup.ContainsKey("DigestUpdate"))
                                    {
                                        CS_DigestUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DigestKey":
                                    if (currentgroup.ContainsKey("DigestKey"))
                                    {
                                        CS_DigestKey();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DigestFinal":
                                    if (currentgroup.ContainsKey("DigestFinal"))
                                    {
                                        CS_DigestFinal();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SignInit":
                                    if (currentgroup.ContainsKey("SignInit"))
                                    {
                                        CS_SignInit();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_Sign":
                                    if (currentgroup.ContainsKey("Sign"))
                                    {
                                        CS_Sign();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CompactSign":
                                    if (currentgroup.ContainsKey("CompactSign"))
                                    {
                                        CS_CompactSign();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SignUpdate":
                                    if (currentgroup.ContainsKey("SignUpdate"))
                                    {
                                        CS_SignUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SignFinal":
                                    if (currentgroup.ContainsKey("SignFinal"))
                                    {
                                        CS_SignFinal();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SignRecoverInit":
                                    if (currentgroup.ContainsKey("SignRecoverInit"))
                                    {
                                        CS_SignRecoverInit();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SignRecover":
                                    if (currentgroup.ContainsKey("SignRecover"))
                                    {
                                        CS_SignRecover();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;    
                                    break;
                                case "DKMS_CompactSignRecover":
                                    if (currentgroup.ContainsKey("CompactSignRecover"))
                                    {
                                        CS_CompactSignRecover();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_VerifyInit":
                                    if (currentgroup.ContainsKey("VerifyInit"))
                                    {
                                        CS_VerifyInit();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_Verify":
                                    if (currentgroup.ContainsKey("Verify"))
                                    {
                                        CS_Verify();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CompactVerify":
                                    if (currentgroup.ContainsKey("CompactVerify"))
                                    {
                                        CS_CompactVerify();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_VerifyUpdate":
                                    if (currentgroup.ContainsKey("VerifyUpdate"))
                                    {
                                        CS_VerifyUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_VerifyFinal":
                                    if (currentgroup.ContainsKey("VerifyFinal"))
                                    {
                                        CS_VerifyFinal();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_VerifyRecoverInit":
                                    if (currentgroup.ContainsKey("VerifyRecoverInit"))
                                    {
                                        CS_VerifyRecoverInit();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_VerifyRecover":
                                    if (currentgroup.ContainsKey("VerifyRecover"))
                                    {
                                        CS_VerifyRecover();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_CompactVerifyRecover":
                                    if (currentgroup.ContainsKey("CompactVerifyRecover"))
                                    {
                                        CS_CompactVerifyRecover();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DigestEncryptUpdate":
                                    if (currentgroup.ContainsKey("DigestEncryptUpdate"))
                                    {
                                        CS_DigestEncryptUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DecryptDigestUpdate":
                                    if (currentgroup.ContainsKey("DecryptDigestUpdate"))
                                    {
                                        CS_DecryptDigestUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SignEncryptUpdate":
                                    if (currentgroup.ContainsKey("SignEncryptUpdate"))
                                    {
                                        CS_SignEncryptUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;    
                                    break;
                                case "DKMS_DecryptVerifyUpdate":
                                    if (currentgroup.ContainsKey("DecryptVerifyUpdate"))
                                    {
                                        CS_DecryptVerifyUpdate();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GenerateKey":
                                    if (currentgroup.ContainsKey("GenerateKey"))
                                    {
                                        CS_GenerateKey();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GenerateKeyPair":
                                    if (currentgroup.ContainsKey("GenerateKeyPair"))
                                    {
                                        CS_GenerateKeyPair();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_WrapKey":
                                    if (currentgroup.ContainsKey("WrapKey"))
                                    {
                                        CS_WrapKey();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_UnwrapKey":
                                    if (currentgroup.ContainsKey("UnwrapKey"))
                                    {
                                        CS_UnwrapKey();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_DeriveKey":
                                    if (currentgroup.ContainsKey("DeriveKey"))
                                    {
                                        CS_DeriveKey();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SetAttributeValue":
                                    if (currentgroup.ContainsKey("SetAttributeValue"))
                                    {
                                        CS_SetAttributeValue();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetObjectSize":
                                    if (currentgroup.ContainsKey("GetObjectSize"))
                                    {
                                        CS_GetObjectSize();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SetPIN":
                                    if (currentgroup.ContainsKey("SetPIN"))
                                    {
                                        CS_SetPIN();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_InitPIN":
                                    if (currentgroup.ContainsKey("InitPIN"))
                                    {
                                        CS_InitPIN();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_InitToken":
                                    if (currentgroup.ContainsKey("InitToken"))
                                    {
                                        CS_InitToken();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetMechanismInfo":
                                    if (currentgroup.ContainsKey("GetMechanismInfo"))
                                    {
                                        CS_GetMechanismInfo();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetSlotInfo":
                                    if (currentgroup.ContainsKey("GetSlotInfo"))
                                    {
                                        CS_GetSlotInfo();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_GetSlotList":
                                    if (currentgroup.ContainsKey("GetSlotList"))
                                    {
                                        CS_GetSlotList();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_TestDeviceComunication":
                                    if (currentgroup.ContainsKey("TestDeviceComunication"))
                                    {
                                        CS_TestDeviceComunication();
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                case "DKMS_SetPolicy":
                                    if (currentgroup.ContainsKey("SetPolicy"))
                                    {
                                        lock (_c.adminsgc.SyncAGContainer.SyncRoot)
                                        {
                                            lock (_c.controllersgc.SyncCGContainer.SyncRoot)
                                            {
                                                lock (_c.custodiansgc.SyncCGContainer.SyncRoot)
                                                {
                                                    lock (_c.insertersgc.SyncIGContainer.SyncRoot)
                                                    {
                                                        lock (_c.operatorsgc.SyncOGContainer.SyncRoot)
                                                        {
                                                            lock (_c.swadminsgc.SyncSGContainer.SyncRoot)
                                                            {
                                                                lock (_c.swdevelopersgc.SyncSGContainer.SyncRoot)
                                                                {
                                                                    lock (_c.techniciansgc.SyncTGContainer.SyncRoot)
                                                                    {
                                                                        CS_SetPolicy();
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    else
                                        ret = ErrorManager.DKMS_ACCESS_DENIED;
                                    break;
                                // Function not found
                                default:
                                    ret = ErrorManager.DKMS_FUNCTION_NOT_FOUND;                                    
                                    break;
                            }

                            // Fill the output buffer if error is found
                            if (ret != ErrorManager.DKMS_OK)
                                _pm.XMLTXParser_Set_Error(ret);

                            // Fill the output buffer
                            TxProtocolBlock = _pm.TxProtocolBlock;
                        }
                    }
				}
			}
		}			
			
		/**
		* This is a function used in order to open a session between an application and
		* a token in a particular slot					  	 
		*/
		private void CS_OpenSession()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              slotID;
			CryptoCore.Smf.SESSION_INFO_FLAGS	flags;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 slotID, SESSION_INFO_FLAGS flags}            // 
			/////////////////////////////////////////////////////////////////
			/* get slotID */ 
			slotID = UInt32.Parse(((ProtocolManager.Param)config.body.param["slotID"]).vvalue);
			/* get flags */
			flags = (CryptoCore.Smf.SESSION_INFO_FLAGS)int.Parse(((ProtocolManager.Param)config.body.param["flags"]).vvalue);
                
			// Send Command - OpenSession
			ret = _smf.OpenSession(slotID, flags);			
    			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
                // Increase the counter in the container
                lock (_c.csc.SyncCSContainer.SyncRoot)
                {
                    UInt32 ss = (UInt32)_c.csc.SyncCSContainer["sessions"];
                    ss++;
                    _c.csc.SyncCSContainer["sessions"] = ss;
                }

				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_OpenSession", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "CurrentSession", ProtocolManager.PARAM_TYPE.UINT32, _smf.CurrentSession.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}

		/**
		* This is a function used in order to close a session between an application and
		* a token					 
		*/
		private void CS_CloseSession()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession}                                    // 
			/////////////////////////////////////////////////////////////////
			/* get hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);

			// Send Command - CloseSession			
			ret = _smf.CloseSession(hSession);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
                // Decrease the counter in the container
                lock (_c.csc.SyncCSContainer.SyncRoot)
                {
                    UInt32 ss = (UInt32)_c.csc.SyncCSContainer["sessions"];
                    if (ss > 0)
                    {
                        ss--;
                        _c.csc.SyncCSContainer["sessions"] = ss;
                    }
                    if (ss == 0)
                    {
                        _cc.Finalize();
                        _cc.Initialize(true); 
                    }
                }

				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_CloseSession", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}		

		/**
		* This is a function used in order to obtain information about a session		
		*/
		private void CS_GetSessionInfo()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			// Function arguments OUT
			CryptoCore.Smf.CK_SESSION_INFO      pInfo;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, out CK_SESSION_INFO pInfo}         // 
			/////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;

			////////////////////////////////////////////////
			// Initialization                             //
			////////////////////////////////////////////////			
			/* initialize pInfo */
			pInfo = new CryptoCore.Smf.CK_SESSION_INFO();

			// Send Command - GetSessionInfo
			ret = _smf.GetSessionInfo(out pInfo);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetSessionInfo", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_StartParamStruct(1, "pInfo", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(0, "pInfo.flags", ProtocolManager.PARAM_TYPE.UINT32, pInfo.flags.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.slotID", ProtocolManager.PARAM_TYPE.UINT32, pInfo.slotID.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(2, "pInfo.state", ProtocolManager.PARAM_TYPE.UINT32, pInfo.state.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(3, "pInfo.ulDeviceError", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulDeviceError.ToString());
				_pm.XMLTXParser_Set_EndParamStruct();
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}

		/**
		* This is a function used in order to log a user into a token		
		*/
		private void CS_Login()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;				
			// Function arguments IN
			UInt32                              hSession;
			CryptoCore.Smf.CK_USER_TYPE         userType;
			string                              Pin;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, UInt32 userType, string Pin}       // 
			/////////////////////////////////////////////////////////////////
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get userType */
			userType = (CryptoCore.Smf.CK_USER_TYPE)int.Parse(((ProtocolManager.Param)config.body.param["userType"]).vvalue);
			/* get Pin */
			Pin = ((ProtocolManager.Param)config.body.param["Pin"]).vvalue;			

			// Send Command - Login
			ret = _smf.Login(userType, Pin);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_Login", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}			
		}

		/**
		* This is a function used in order to log a user out from a token		
		*/
		private void CS_Logout()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32								hSession;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession}                                    // 
			/////////////////////////////////////////////////////////////////		
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;

			// Send Command - Logout
			ret = _smf.Logout();

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_Logout", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}			
		}

		/**
		* This is a function used in order to create a new object		
		*/
		private void CS_CreateObject()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;
			UInt32								ulCount;
			// Function arguments IN
			UInt32                              hSession;
			CryptoCore.ATTRIBUTE_VALUE          templ;
			// Function arguments OUT				
			UInt32                              phObject;


			// Initialization
			converter = new CryptoCore.CCConvert();

			//////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                       //
			// Param: {ATTRIBUTE_VALUE templ, out UInt32 phObject}				// 
			//////////////////////////////////////////////////////////////////////				
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get templ */
			ulCount = ((ProtocolManager.Param)config.body.param["templ"]).XDim;			
			ret = converter.ParseAttributeIn(config, "templ", ulCount, out templ);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - CreateObject
				ret = _smf.omf.CreateObject(templ, out phObject);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);				
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_CreateObject", 2);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					_pm.XMLTXParser_Set_ParamAtomic(1, "phObject", ProtocolManager.PARAM_TYPE.UINT32, phObject.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}
		
		/**
		* This is a function used in order to initialize a search for token and session objects
		* that match a template		
		*/
		private void CS_FindObjectsInit()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;
			UInt32								ulCount;
			// Function arguments IN
			UInt32                              hSession;
			CryptoCore.ATTRIBUTE_VALUE          templ;

			
			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, CryptoCore.ATTRIBUTE_VALUE templ}  // 
			/////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get templ */	
			ulCount = ((ProtocolManager.Param)config.body.param["templ"]).XDim;			
			ret = converter.ParseAttributeIn(config, "templ", ulCount, out templ);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - FindObjectsInit
				ret = _smf.omf.FindObjectsInit(templ);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);				
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_FindObjectsInit", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}			
			}
		}
			
		/**
		* This is a function used in order to continue a search for token and session objects that match a template,
		* obtaining additional object handles		
		*/
		private void CS_FindObjects()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			int                                 maxObjectNumber;
			// Function arguments OUT				
			UInt32[]                            hObject;


			//////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                           //
			// Param: {UInt32 hSession, int maxObjectNumber, out UInt32[] hObject}  // 
			//////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get maxObjectNumber */
			maxObjectNumber = int.Parse(((ProtocolManager.Param)config.body.param["maxObjectNumber"]).vvalue);

			// Send Command - FindObjects
			ret = _smf.omf.FindObjects(out hObject, maxObjectNumber);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_FindObjects", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_StartParamArray(1, "hObject", ProtocolManager.PARAM_TYPE.UINT32, (UInt32)hObject.Length, 0);
				for (int i = 0; i < hObject.Length; i++)
					_pm.XMLTXParser_Set_ParamAtomic((UInt32)i, "hObject[" + i.ToString() + "]", ProtocolManager.PARAM_TYPE.UINT32, hObject[i].ToString());
				_pm.XMLTXParser_Set_EndParamArray();
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}			
		}

		/**
		* This is a function used in order to terminate a search for token and session objects		
		*/
		private void CS_FindObjectsFinal()
		{
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession}                                    // 
			/////////////////////////////////////////////////////////////////					
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;			

			// Send Command - FindObjectsFinal
			ret = _smf.omf.FindObjectsFinal();

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_FindObjectsFinal", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}			
		}

        /**
		* This is a function used in order to search for token and session objects that match a template,
		* obtaining additional object handles		
		*/
        private void CS_CompactFindObjects()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            CryptoCore.CCConvert				converter;
			UInt32								ulCount;
			// Function arguments IN
            UInt32                              hSession;
            int                                 maxObjectNumber;			
			CryptoCore.ATTRIBUTE_VALUE          templ;            
            // Function arguments OUT				
            UInt32[]                            hObject;


            // Initialization
            converter = new CryptoCore.CCConvert();

            //////////////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                           //
            // Param: {UInt32 hSession, CryptoCore.ATTRIBUTE_VALUE templ,           //
            // int maxObjectNumber, out UInt32[] hObject}                           // 
            //////////////////////////////////////////////////////////////////////////			
            /* get-set hSession */
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;            
            /* get maxObjectNumber */
            maxObjectNumber = int.Parse(((ProtocolManager.Param)config.body.param["maxObjectNumber"]).vvalue);
             /* get templ */	
			ulCount = ((ProtocolManager.Param)config.body.param["templ"]).XDim;			
			ret = converter.ParseAttributeIn(config, "templ", ulCount, out templ);
            		
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
                // Send Command - CompactFindObjects
                ret = _smf.omf.CompactFindObjects(templ, out hObject, maxObjectNumber);

				// Bad Result
                if (ret != ErrorManager.DKMS_OK)
                {
                    _pm.XMLTXParser_Set_Error(ret);
                }
                // Good Result
                else
                {
                    //////////////////////////////////////
                    // Fase 3 Build the responce					
                    /* 1 */
                    _pm.XMLTXParser_Set_Start("DKMS_CompactFindObjects", 2);
                    /* 2 */
                    _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                    _pm.XMLTXParser_Set_StartParamArray(1, "hObject", ProtocolManager.PARAM_TYPE.UINT32, (UInt32)hObject.Length, 0);
                    for (int i = 0; i < hObject.Length; i++)
                        _pm.XMLTXParser_Set_ParamAtomic((UInt32)i, "hObject[" + i.ToString() + "]", ProtocolManager.PARAM_TYPE.UINT32, hObject[i].ToString());
                    _pm.XMLTXParser_Set_EndParamArray();
                    /* 3 */
                    _pm.XMLTXParser_Set_End();                        
                }
            }
        }

		/**
		* This is a function used in order to initialize an encryption operation		
		*/
		private void CS_EncryptInit()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;		
			// Function arguments IN
			UInt32                              hSession;				
			UInt32                              hKey;
			CryptoCore.Smf.MECHANISM            mech;

            
			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey}       // 
			/////////////////////////////////////////////////////////////////
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hKey */ 
			hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
			/* get mech */
			ret = converter.ParseMechanismIn(out mech, config);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - EncryptInit
				ret = _smf.ef.EncryptInit(mech, hKey);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);					
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_EncryptInit", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}
			
		/**
		* This is a function used in order to encrypt single-part data		
		*/
		private void CS_Encrypt()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              ClearData;
			// Function arguments OUT				
			string                              EncryptedData;


			//////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                           //
			// Param: {UInt32 hSession, string ClearData, out string EncryptedData} // 
			//////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get ClearData */
			ClearData = ((ProtocolManager.Param)config.body.param["ClearData"]).vvalue;
			
			// Send Command - Encrypt
			ret = _smf.ef.Encrypt(ClearData, out EncryptedData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_Encrypt", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "EncryptedData", ProtocolManager.PARAM_TYPE.STRING, EncryptedData);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

        /**
        * This is a function used in order to initialize an encryption operation and encrypt single-part data		
        */
        private void CS_CompactEncrypt()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            CryptoCore.CCConvert                converter;
            // Function arguments IN
            UInt32                              hSession;
            UInt32                              hKey;
            CryptoCore.Smf.MECHANISM            mech;
            string                              ClearData;
            // Function arguments OUT				
            string                              EncryptedData;


            // Initialization
            converter = new CryptoCore.CCConvert();

            /////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                  //
            // Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey,       // 
            // string ClearData, out string EncryptedData}                 // 
            /////////////////////////////////////////////////////////////////
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
            /* get ClearData */
            ClearData = ((ProtocolManager.Param)config.body.param["ClearData"]).vvalue;
            /* get mech */
            ret = converter.ParseMechanismIn(out mech, config);

            // Bad Result
            if (ret != ErrorManager.DKMS_OK)
            {
                _pm.XMLTXParser_Set_Error(ret);
            }
            // Good Result
            else
            {
                // Send Command - CompactEncrypt
                ret = _smf.ef.CompactEncrypt(mech, hKey, ClearData, out EncryptedData);

                // Bad Result
                if (ret != ErrorManager.DKMS_OK)
                {
                    _pm.XMLTXParser_Set_Error(ret);
                }
                // Good Result
                else
                {                    
                    //////////////////////////////////////
                    // Fase 3 Build the responce
                    /* 1 */
                    _pm.XMLTXParser_Set_Start("DKMS_CompactEncrypt", 2);
                    /* 2 */
                    _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                    _pm.XMLTXParser_Set_ParamAtomic(1, "EncryptedData", ProtocolManager.PARAM_TYPE.STRING, EncryptedData);
                    /* 3 */
                    _pm.XMLTXParser_Set_End();                    
                }
            }
        }

		/**
		* This is a function used in order to continue a multiple-part encryption operation,
		* processing another data part		
		*/
		private void CS_EncryptUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              ClearPartData;
			// Function arguments OUT				
			string                              EncryptedPartData;


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                                   //
			// Param: {UInt32 hSession, string ClearPartData, out string EncryptedPartData} // 
			//////////////////////////////////////////////////////////////////////////////////
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get ClearPartData */
			ClearPartData = ((ProtocolManager.Param)config.body.param["ClearPartData"]).vvalue;
			
			// Send Command - EncryptUpdate
			ret = _smf.ef.EncryptUpdate(ClearPartData, out EncryptedPartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_EncryptUpdate", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "EncryptedPartData", ProtocolManager.PARAM_TYPE.STRING, EncryptedPartData);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to finish a multiple-part encryption operation		
		*/
		private void CS_EncryptFinal()
		{				
			UInt32 ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			// Function arguments OUT
			string                              LastEncryptedPartData;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, out string LastEncryptedPartData}  // 
			/////////////////////////////////////////////////////////////////		
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			
			// Send Command - EncryptFinal
			ret = _smf.ef.EncryptFinal(out LastEncryptedPartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_EncryptFinal", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "LastEncryptedPartData", ProtocolManager.PARAM_TYPE.STRING, LastEncryptedPartData);
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}
		}
			
		/**
		* This is a function used in order to initialize a decryption operation		
		*/
		private void CS_DecryptInit()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;		
			// Function arguments IN
			UInt32                              hSession;			
			CryptoCore.Smf.MECHANISM            mech;
			UInt32                              hKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey}       // 
			/////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hKey */
			hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
			/* get mech */
			ret = converter.ParseMechanismIn(out mech, config);			

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - DecryptInit
				ret = _smf.df.DecryptInit(mech, hKey);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);					
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_DecryptInit", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();					
				}
			}
		}
				
		/**
		* This is a function used in order to decrypt single-part data		
		*/
		private void CS_Decrypt()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;					
			// Function arguments IN
			UInt32                              hSession;				
			string                              EncryptedData;
			// Function arguments OUT				
			string                              ClearData;


			//////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                           //
			// Param: {UInt32 hSession, string EncryptedData, out string ClearData} // 
			//////////////////////////////////////////////////////////////////////////	
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get EncryptedData */
			EncryptedData = ((ProtocolManager.Param)config.body.param["EncryptedData"]).vvalue;
			
			// Send Command - Decrypt
			ret = _smf.df.Decrypt(EncryptedData, out ClearData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_Decrypt", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "ClearData", ProtocolManager.PARAM_TYPE.STRING, ClearData);
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}
		}

        /**
		* This is a function used in order to initialize a decryption operation	and for decrypt single-part data	
		*/
        private void CS_CompactDecrypt()
        {
            UInt32 ret = ErrorManager.DKMS_OK;
            CryptoCore.CCConvert converter;
            // Function arguments IN
            UInt32 hSession;
            CryptoCore.Smf.MECHANISM mech;
            UInt32 hKey;
            string EncryptedData;
            // Function arguments OUT				
            string ClearData;


            // Initialization
            converter = new CryptoCore.CCConvert();

            /////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                  //
            // Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey,       // 
            // string EncryptedData, out string ClearData}                 // 
            /////////////////////////////////////////////////////////////////			
            /* get-set hSession */
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
            /* get EncryptedData */
            EncryptedData = ((ProtocolManager.Param)config.body.param["EncryptedData"]).vvalue;
            /* get mech */
            ret = converter.ParseMechanismIn(out mech, config);

            // Bad Result
            if (ret != ErrorManager.DKMS_OK)
            {
                _pm.XMLTXParser_Set_Error(ret);
            }
            // Good Result
            else
            {
                // Send Command - CompactDecrypt
                ret = _smf.df.CompactDecrypt(mech, hKey, EncryptedData, out ClearData);

                // Bad Result
                if (ret != ErrorManager.DKMS_OK)
                {
                    _pm.XMLTXParser_Set_Error(ret);
                }
                // Good Result
                else
                {                    
                    //////////////////////////////////////
                    // Fase 3 Build the responce					
                    /* 1 */
                    _pm.XMLTXParser_Set_Start("DKMS_CompactDecrypt", 2);
                    /* 2 */
                    _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                    _pm.XMLTXParser_Set_ParamAtomic(1, "ClearData", ProtocolManager.PARAM_TYPE.STRING, ClearData);
                    /* 3 */
                    _pm.XMLTXParser_Set_End();                    
                }
            }
        }
				
		/**
		* This is a function used in order to continue a multiple-part decryption operation,
		* processing another encrypted data part		
		*/
		private void CS_DecryptUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              EncryptedPartData;
			// Function arguments OUT				
			string                              ClearPartData;


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                                   //
			// Param: {UInt32 hSession, string EncryptedPartData, out string ClearPartData} // 
			//////////////////////////////////////////////////////////////////////////////////
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get EncryptedPartData */
			EncryptedPartData = ((ProtocolManager.Param)config.body.param["EncryptedPartData"]).vvalue;
			
			// Send Command - DecryptUpdate
			ret = _smf.df.DecryptUpdate(EncryptedPartData, out ClearPartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DecryptUpdate", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "ClearPartData", ProtocolManager.PARAM_TYPE.STRING, ClearPartData);
				/* 3 */
				_pm.XMLTXParser_Set_End();			
			}
		}
			
		/**
		* This is a function used in order to finish a multiple-part decryption operation		
		*/
		private void CS_DecryptFinal()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;				
			// Function arguments IN
			UInt32                              hSession;
			// Function arguments OUT
			string                              LastDecryptedPartData;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, out string LastDecryptedPartData}  // 
			/////////////////////////////////////////////////////////////////			
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			
			// Send Command - DecryptFinal
			ret = _smf.df.DecryptFinal(out LastDecryptedPartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DecryptFinal", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "LastDecryptedPartData", ProtocolManager.PARAM_TYPE.STRING, LastDecryptedPartData);
				/* 3 */
				_pm.XMLTXParser_Set_End();		
			}
		}

		/**
		* This is a function used in order to returns general information about Cryptoki		
		*/
		private void CS_GetInfo()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;						
			// Function arguments OUT
			CryptoCore.CK_INFO					pInfo;
			

			// Send Command - GetInfo
			ret = _cc.GetInfo(out pInfo);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetInfo", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_StartParamStruct(1, "pInfo", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_StartParamStruct(0, "pInfo.cryptokiVersion", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(0, "pInfo.cryptokiVersion.major", ProtocolManager.PARAM_TYPE.BYTE, pInfo.cryptokiVersion.major.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.cryptokiVersion.minor", ProtocolManager.PARAM_TYPE.BYTE, pInfo.cryptokiVersion.minor.ToString());
				_pm.XMLTXParser_Set_EndParamStruct();
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.manufacturerID", ProtocolManager.PARAM_TYPE.STRING, pInfo.manufacturerID);
				_pm.XMLTXParser_Set_ParamAtomic(2, "pInfo.flags", ProtocolManager.PARAM_TYPE.UINT32, pInfo.flags.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(3, "pInfo.libraryDescription", ProtocolManager.PARAM_TYPE.STRING, pInfo.libraryDescription);
				_pm.XMLTXParser_Set_StartParamStruct(4, "pInfo.libraryVersion", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(0, "pInfo.libraryVersion.major", ProtocolManager.PARAM_TYPE.BYTE, pInfo.libraryVersion.major.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.libraryVersion.minor", ProtocolManager.PARAM_TYPE.BYTE, pInfo.libraryVersion.minor.ToString());
				_pm.XMLTXParser_Set_EndParamStruct();
				_pm.XMLTXParser_Set_EndParamStruct();
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}
			
		/**
		* This is a function used in order to obtains information about a particular token in the system		
		*/
		private void CS_GetTokenInfo()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              slotID;
			// Function arguments OUT
			CryptoCore.Stmf.CK_TOKEN_INFO       pInfo;


			//////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                       //
			// Param: {UInt32 hSession, UInt32 slotID, out CK_TOKEN_INFO pInfo} // 
			//////////////////////////////////////////////////////////////////////			
			slotID = UInt32.Parse(((ProtocolManager.Param)config.body.param["slotID"]).vvalue);
			
			// Send Command - GetTokenInfo                
			ret =  _stmf.GetTokenInfo(slotID, out pInfo);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetTokenInfo", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_StartParamStruct(1, "pInfo", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(0, "pInfo.label", ProtocolManager.PARAM_TYPE.STRING, pInfo.label);
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.manufacturerID", ProtocolManager.PARAM_TYPE.STRING, pInfo.manufacturerID);
				_pm.XMLTXParser_Set_ParamAtomic(2, "pInfo.model", ProtocolManager.PARAM_TYPE.STRING, pInfo.model);
				_pm.XMLTXParser_Set_ParamAtomic(3, "pInfo.serialNumber", ProtocolManager.PARAM_TYPE.STRING, pInfo.serialNumber);
				_pm.XMLTXParser_Set_ParamAtomic(4, "pInfo.flags", ProtocolManager.PARAM_TYPE.UINT32, pInfo.flags.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(5, "pInfo.ulMaxSessionCount", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulMaxSessionCount.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(6, "pInfo.ulSessionCount", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulSessionCount.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(7, "pInfo.ulMaxRwSessionCount", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulMaxRwSessionCount.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(8, "pInfo.ulRwSessionCount", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulRwSessionCount.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(9, "pInfo.ulMaxPinLen", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulMaxPinLen.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(10, "pInfo.ulMinPinLen", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulMinPinLen.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(11, "pInfo.ulTotalPublicMemory", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulTotalPublicMemory.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(12, "pInfo.ulFreePublicMemory", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulFreePublicMemory.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(13, "pInfo.ulTotalPrivateMemory", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulTotalPrivateMemory.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(14, "pInfo.ulFreePrivateMemory", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulFreePrivateMemory.ToString());
				_pm.XMLTXParser_Set_StartParamStruct(15, "pInfo.hardwareVersion", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(0, "pInfo.hardwareVersion.major", ProtocolManager.PARAM_TYPE.BYTE, pInfo.hardwareVersion.major.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.hardwareVersion.minor", ProtocolManager.PARAM_TYPE.BYTE, pInfo.hardwareVersion.minor.ToString());
				_pm.XMLTXParser_Set_EndParamStruct();
				_pm.XMLTXParser_Set_StartParamStruct(16, "pInfo.firmwareVersion", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(0, "pInfo.firmwareVersion.major", ProtocolManager.PARAM_TYPE.BYTE, pInfo.firmwareVersion.major.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.firmwareVersion.minor", ProtocolManager.PARAM_TYPE.BYTE, pInfo.firmwareVersion.minor.ToString());
				_pm.XMLTXParser_Set_EndParamStruct();
				_pm.XMLTXParser_Set_ParamAtomic(17, "pInfo.utcTime", ProtocolManager.PARAM_TYPE.STRING, pInfo.utcTime);
				_pm.XMLTXParser_Set_EndParamStruct();
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}			
		}

		/**
		* This is a function used in order to mix additional seed material into the tokens
		* random number generator		
		*/
		private void CS_SeedRandom()
		{               
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			string                              Seed;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, string Seed}                       // 
			/////////////////////////////////////////////////////////////////
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get Seed */
			Seed = ((ProtocolManager.Param)config.body.param["Seed"]).vvalue;			

			// Send Command - SeedRandom
			ret = _smf.rngf.SeedRandom(Seed);

			// Bad Result
			if (ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_SeedRandom", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to generate random or pseudo-random data		
		*/
		private void CS_GenerateRandom()
		{               
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			UInt32                              ulRandomLen;
			// Function arguments OUT
			string                              RandomData;


			//////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                           //
			// Param: {UInt32 hSession, out string RandomData, UInt32 ulRandomLen}  // 
			//////////////////////////////////////////////////////////////////////////
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get ulRandomLen */
			ulRandomLen = UInt32.Parse(((ProtocolManager.Param)config.body.param["ulRandomLen"]).vvalue);
			
			// Send Command - GenerateRandom
			ret = _smf.rngf.GenerateRandom(out RandomData, ulRandomLen);

			// Bad Result
			if (ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GenerateRandom", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "RandomData", ProtocolManager.PARAM_TYPE.STRING, RandomData);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

        /**
		* This is a function used in order to mix additional seed material into the tokens
		* random number generator and generate random or pseudo-random data		
		*/
        private void CS_CompactGenerateRandom()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            // Function arguments IN
            UInt32                              hSession;
            string                              Seed;
            UInt32                              ulRandomLen;
            // Function arguments OUT
            string                              RandomData;


            /////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                  //
            // Param: {UInt32 hSession, string Seed,                       // 
            // out string RandomData, UInt32 ulRandomLen}                  // 
            /////////////////////////////////////////////////////////////////
            /* get-set hSession */
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;
            /* get ulRandomLen */
            ulRandomLen = UInt32.Parse(((ProtocolManager.Param)config.body.param["ulRandomLen"]).vvalue);
            /* get Seed */
            Seed = ((ProtocolManager.Param)config.body.param["Seed"]).vvalue;

            // Send Command - CompactGenerateRandom
            ret = _smf.rngf.CompactGenerateRandom(Seed, out RandomData, ulRandomLen);

            // Bad Result
            if (ret != ErrorManager.DKMS_OK)
            {
                _pm.XMLTXParser_Set_Error(ret);
            }
            // Good Result
            else
            {                
                //////////////////////////////////////
                // Fase 3 Build the responce					
                /* 1 */
                _pm.XMLTXParser_Set_Start("DKMS_CompactGenerateRandom", 2);
                /* 2 */
                _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                _pm.XMLTXParser_Set_ParamAtomic(1, "RandomData", ProtocolManager.PARAM_TYPE.STRING, RandomData);
                /* 3 */
                _pm.XMLTXParser_Set_End();                
            }
        }

		/**
		*  This is a function used in order to generate a secret key, creating a new key object		
		*/
		private void CS_GenerateKey()
		{                
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;
			UInt32								ulCount;
			// Function arguments IN
			UInt32                              hSession;
			CryptoCore.ATTRIBUTE_VALUE          templ;
			CryptoCore.Smf.MECHANISM            mech;
			// Function arguments OUT				
			UInt32                              phKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			///////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                                    //
			// Param: {UInt32 hSession, CryptoCore.ATTRIBUTE_VALUE templ, MECHANISM mech,    //
			// out UInt32 phKey}															 // 
			///////////////////////////////////////////////////////////////////////////////////		
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get templ */ 
			ulCount = ((ProtocolManager.Param)config.body.param["templ"]).XDim;			
			ret = converter.ParseAttributeIn(config, "templ", ulCount, out templ);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{			
				/* get mech */
				ret = converter.ParseMechanismIn(out mech, config);

				// Bad Result
				if (ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);				
				}
				// Good Result
				else
				{
					// Send Command - GenerateKey
					ret = _smf.kmf.GenerateKey(templ, mech, out phKey);

					// Bad Result
					if (ret != ErrorManager.DKMS_OK)
					{
						_pm.XMLTXParser_Set_Error(ret);					
					}
					// Good Result
					else
					{
						//////////////////////////////////////
						// Fase 3 Build the responce					
						/* 1 */
						_pm.XMLTXParser_Set_Start("DKMS_GenerateKey", 2);
						/* 2 */
						_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
						_pm.XMLTXParser_Set_ParamAtomic(1, "phKey", ProtocolManager.PARAM_TYPE.UINT32, phKey.ToString());
						/* 3 */
						_pm.XMLTXParser_Set_End();
					}
				}
			}
		}

		/**
		* This is a function used in order to destroy an object		
		*/
		private void CS_DestroyObject()
		{                
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession; 
			UInt32                              hObject;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, UInt32 hObject}                    // 
			/////////////////////////////////////////////////////////////////
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hObject */
			hObject = UInt32.Parse(((ProtocolManager.Param)config.body.param["hObject"]).vvalue);

			// Send Command - DestroyObject
			ret = _smf.omf.DestroyObject(hObject);

			// Bad Result
			if (ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DestroyObject", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to obtain a list of mechanism types supported by a token		
		*/
		private void CS_GetMechanismList()
		{                
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN                
			UInt32                              slotID;
			// Function arguments OUT				
			Int32[]                             MechanismList;


			///////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                //
			// Param: {UInt32 slotID, out Int32[] MechanismList}         // 
			///////////////////////////////////////////////////////////////         
			/* get slotID */ 
			slotID = UInt32.Parse(((ProtocolManager.Param)config.body.param["slotID"]).vvalue);

			// Send Command - GetMechanismList                
			ret = _stmf.GetMechanismList(slotID, out MechanismList);

			// Bad Result
			if (ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetMechanismList", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_StartParamArray(1, "MechanismList", ProtocolManager.PARAM_TYPE.INT32, (UInt32)MechanismList.Length, 0);
				for (int i = 0; i < MechanismList.Length; i++)
					_pm.XMLTXParser_Set_ParamAtomic((UInt32)i, "MechanismList[" + i.ToString() + "]", ProtocolManager.PARAM_TYPE.INT32, MechanismList[i].ToString());
				_pm.XMLTXParser_Set_EndParamArray();
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to obtain the value of one or more attributes of an object		
		*/
		private void CS_GetAttributeValue()
		{                
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;
			UInt32								ulCount;
			// function arguments in
			UInt32                              hSession;
			UInt32                              hObject;                			
			CryptoCore.CK_ATTRIBUTE_TYPE[]      atype;
			// function arguments out
			CryptoCore.ATTRIBUTE_VALUE			templ;


			// Initialization
			converter = new CryptoCore.CCConvert();

			//////////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block										//
			// Param: {UInt32 hSession, CK_ATTRIBUTE_TYPE[] atype, out ATTRIBUTE_VALUE templ,	//
			// UInt32 hObject}																	// 
			//////////////////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get att_type */
			ulCount = ((ProtocolManager.Param)config.body.param["atype"]).XDim;
			atype = new CryptoCore.CK_ATTRIBUTE_TYPE[ulCount];
			for (int i = 0; i < ulCount; i++)
				atype[i] = (CryptoCore.CK_ATTRIBUTE_TYPE)int.Parse(((ProtocolManager.Param)config.body.param["atype[" + i.ToString() + "]"]).vvalue);
			/* get hObject */
			hObject = UInt32.Parse(((ProtocolManager.Param)config.body.param["hObject"]).vvalue);
						
			// Send Command - GetAttributeValue			
			ret = _smf.omf.GetAttributeValue(atype, out templ, hObject);

			// Bad Result
			if (ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);			
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetAttributeValue", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				// parsing CSM templ			
				ret = converter.ParseAttributeOut(templ, ref _pm, converter.AttributeCount(templ), 1, "templ");	
				if (ret != ErrorManager.DKMS_OK)
					throw new Exception();						
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to initialize a digest operation		
		*/
		private void CS_DigestInit()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;		
			// Function arguments IN
			UInt32                              hSession;			
			CryptoCore.Smf.MECHANISM            mech;


			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, MECHANISM mech}					   // 
			/////////////////////////////////////////////////////////////////
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;			
			/* get mech */
			ret = converter.ParseMechanismIn(out mech, config);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - DigestInit
				ret = _smf.mdf.DigestInit(mech);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);					
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_DigestInit", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}

		/**
		* This is a function used in order to digest data in a single part	
		*/
		private void CS_Digest()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              Data;
			// Function arguments OUT				
			string                              Digest;


			//////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                           //
			// Param: {UInt32 hSession, string Data, out string Digest}				// 
			//////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get Data */
			Data = ((ProtocolManager.Param)config.body.param["Data"]).vvalue;
			
			// Send Command - Digest
			ret = _smf.mdf.Digest(Data, out Digest);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_Digest", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "Digest", ProtocolManager.PARAM_TYPE.STRING, Digest);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

        /**
		* This is a function used in order to initialize a digest operation	and digest data in a single part	
		*/
        private void CS_CompactDigest()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            CryptoCore.CCConvert                converter;
            // Function arguments IN
            UInt32                              hSession;
            CryptoCore.Smf.MECHANISM            mech;
            string                              Data;
            // Function arguments OUT				
            string                              Digest;


            // Initialization
            converter = new CryptoCore.CCConvert();

            /////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                  //
            // Param: {UInt32 hSession, MECHANISM mech,                    // 
            // string Data, out string Digest}					           // 
            /////////////////////////////////////////////////////////////////
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;
            /* get Data */
            Data = ((ProtocolManager.Param)config.body.param["Data"]).vvalue;
            /* get mech */
            ret = converter.ParseMechanismIn(out mech, config);            

            // Bad Result
            if (ret != ErrorManager.DKMS_OK)
            {
                _pm.XMLTXParser_Set_Error(ret);
            }
            // Good Result
            else
            {
                // Send Command - CompactDigest
                ret = _smf.mdf.CompactDigest(mech, Data, out Digest);

                // Bad Result
                if (ret != ErrorManager.DKMS_OK)
                {
                    _pm.XMLTXParser_Set_Error(ret);
                }
                // Good Result
                else
                {                   
                    //////////////////////////////////////
                    // Fase 3 Build the responce
                    /* 1 */
                    _pm.XMLTXParser_Set_Start("DKMS_CompactDigest", 2);
                    /* 2 */
                    _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                    _pm.XMLTXParser_Set_ParamAtomic(1, "Digest", ProtocolManager.PARAM_TYPE.STRING, Digest);
                    /* 3 */
                    _pm.XMLTXParser_Set_End();                    
                }
            }
        }

		/**
		* This is a function used in order to continue a multiple-part message-digesting operation,
		* processing another data part
		*/
		private void CS_DigestUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              PartData;			


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                                   //
			// Param: {UInt32 hSession, string PartData}									// 
			//////////////////////////////////////////////////////////////////////////////////
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get PartData */
			PartData = ((ProtocolManager.Param)config.body.param["PartData"]).vvalue;
			
			// Send Command - DigestUpdate
			ret = _smf.mdf.DigestUpdate(PartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DigestUpdate", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}
		
		/**
		* This is a function used in order to continue a multiple-part message-digesting operation
		* by digesting the value of a secret key		
		*/
		private void CS_DigestKey()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;					
			// Function arguments IN
			UInt32                              hSession;				
			UInt32                              hKey;			


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, UInt32 hKey}					   // 
			/////////////////////////////////////////////////////////////////
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hKey */ 
			hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
						
			// Send Command - DigestKey
			ret = _smf.mdf.DigestKey(hKey);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);					
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DigestKey", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}			
		}

		/**
		* This is a function used in order to finish a multiple-part message-digesting operation,
		* returning the message digest		
		*/
		private void CS_DigestFinal()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			// Function arguments OUT
			string                              Digest;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, out string Digest}				   // 
			/////////////////////////////////////////////////////////////////		
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			
			// Send Command - DigestFinal
			ret = _smf.mdf.DigestFinal(out Digest);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DigestFinal", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "Digest", ProtocolManager.PARAM_TYPE.STRING, Digest);
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}
		}

		/**
		* This is a function used in order to initialize a signature operation,
		* where the signature is an appendix to the data	
		*/
		private void CS_SignInit()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;		
			// Function arguments IN
			UInt32                              hSession;			
			CryptoCore.Smf.MECHANISM            mech;
			UInt32								hKey;

			
			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey}	   // 
			/////////////////////////////////////////////////////////////////
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
			/* get mech */
			ret = converter.ParseMechanismIn(out mech, config);
						
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - SignInit
				ret = _smf.smacf.SignInit(mech, hKey);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);					
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_SignInit", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}

		/**
		* This is a function used in order to sign data in a single part,
		* where the signature is an appendix to the data
		*/
		private void CS_Sign()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              Data;
			// Function arguments OUT				
			string                              Signature;


			//////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                           //
			// Param: {UInt32 hSession, string Data, out string Signature}			// 
			//////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get Data */
			Data = ((ProtocolManager.Param)config.body.param["Data"]).vvalue;
			
			// Send Command - Sign
			ret = _smf.smacf.Sign(Data, out Signature);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_Sign", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "Signature", ProtocolManager.PARAM_TYPE.STRING, Signature);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

        /**
		* This is a function used in order to initialize a signature operation and sign data in a single part,
		* where the signature is an appendix to the data	
		*/
        private void CS_CompactSign()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            CryptoCore.CCConvert                converter;
            // Function arguments IN
            UInt32                              hSession;
            CryptoCore.Smf.MECHANISM            mech;
            UInt32                              hKey;
            string                              Data;
            // Function arguments OUT				
            string                              Signature;


            // Initialization
            converter = new CryptoCore.CCConvert();

            /////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                  //
            // Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey,       //  
            // string Data, out string Signature}                   	   // 
            /////////////////////////////////////////////////////////////////
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;
            /* get Data */
            Data = ((ProtocolManager.Param)config.body.param["Data"]).vvalue;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
            /* get mech */
            ret = converter.ParseMechanismIn(out mech, config);
            
            // Bad Result
            if (ret != ErrorManager.DKMS_OK)
            {
                _pm.XMLTXParser_Set_Error(ret);
            }
            // Good Result
            else
            {
                // Send Command - CompactSign
                ret = _smf.smacf.CompactSign(mech, hKey, Data, out Signature);

                // Bad Result
                if (ret != ErrorManager.DKMS_OK)
                {
                    _pm.XMLTXParser_Set_Error(ret);
                }
                // Good Result
                else
                {                    
                    //////////////////////////////////////
                    // Fase 3 Build the responce
                    /* 1 */
                    _pm.XMLTXParser_Set_Start("DKMS_CompactSign", 2);
                    /* 2 */
                    _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                    _pm.XMLTXParser_Set_ParamAtomic(1, "Signature", ProtocolManager.PARAM_TYPE.STRING, Signature);
                    /* 3 */
                    _pm.XMLTXParser_Set_End();                 
                }
            }
        }

		/**
		* This is a function used in order to continue a multiple-part signature operation,
		* processing another data part
		*/
		private void CS_SignUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              PartData;			


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                                   //
			// Param: {UInt32 hSession, string PartData}									// 
			//////////////////////////////////////////////////////////////////////////////////
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get PartData */
			PartData = ((ProtocolManager.Param)config.body.param["PartData"]).vvalue;
			
			// Send Command - SignUpdate
			ret = _smf.smacf.SignUpdate(PartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_SignUpdate", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to finish a multiple-part signature operation,
		* returning the signature
		*/
		private void CS_SignFinal()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			// Function arguments OUT
			string                              Signature;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, out string Signature}			   // 
			/////////////////////////////////////////////////////////////////		
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			
			// Send Command - SignFinal
			ret = _smf.smacf.SignFinal(out Signature);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_SignFinal", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "Signature", ProtocolManager.PARAM_TYPE.STRING, Signature);
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}
		}

		/**
		* This is a function used in order to initialize a signature operation,
		* where the data can be recovered from the signature	
		*/
		private void CS_SignRecoverInit()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;		
			// Function arguments IN
			UInt32                              hSession;			
			CryptoCore.Smf.MECHANISM            mech;
			UInt32								hKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey}	   // 
			/////////////////////////////////////////////////////////////////
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
			/* get mech */
			ret = converter.ParseMechanismIn(out mech, config);			
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - SignRecoverInit
				ret = _smf.smacf.SignRecoverInit(mech, hKey);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);					
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_SignRecoverInit", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}

		/**
		* This is a function used in order to sign data in a single operation,
		* where the data can be recovered from the signature.
		*/
		private void CS_SignRecover()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			string								Data;
			// Function arguments OUT
			string                              Signature;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, string Data, out string Signature} // 
			/////////////////////////////////////////////////////////////////		
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);			
			_smf.CurrentSession = hSession;
			/* get Data */
			Data = ((ProtocolManager.Param)config.body.param["Data"]).vvalue;
			
			// Send Command - SignRecover
			ret = _smf.smacf.SignRecover(Data, out Signature);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_SignRecover", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "Signature", ProtocolManager.PARAM_TYPE.STRING, Signature);
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}
		}

        /**
		* This is a function used in order to initialize a signature operation and sign data in a single operation,
		* where the data can be recovered from the signature	
		*/
        private void CS_CompactSignRecover()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            CryptoCore.CCConvert                converter;
            // Function arguments IN
            UInt32                              hSession;
            CryptoCore.Smf.MECHANISM            mech;
            UInt32                              hKey;
            string                              Data;
            // Function arguments OUT
            string                              Signature;


            // Initialization
            converter = new CryptoCore.CCConvert();

            /////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                  //
            // Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey,       // 
            // string Data, out string Signature}                   	   // 
            /////////////////////////////////////////////////////////////////
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;
            /* get Data */
            Data = ((ProtocolManager.Param)config.body.param["Data"]).vvalue;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
            /* get mech */
            ret = converter.ParseMechanismIn(out mech, config);            

            // Bad Result
            if (ret != ErrorManager.DKMS_OK)
            {
                _pm.XMLTXParser_Set_Error(ret);
            }
            // Good Result
            else
            {
                // Send Command - CompactSignRecover
                ret = _smf.smacf.CompactSignRecover(mech, hKey, Data, out Signature);

                // Bad Result
                if (ret != ErrorManager.DKMS_OK)
                {
                    _pm.XMLTXParser_Set_Error(ret);
                }
                // Good Result
                else
                {                    
                    //////////////////////////////////////
                    // Fase 3 Build the responce					
                    /* 1 */
                    _pm.XMLTXParser_Set_Start("DKMS_CompactSignRecover", 2);
                    /* 2 */
                    _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                    _pm.XMLTXParser_Set_ParamAtomic(1, "Signature", ProtocolManager.PARAM_TYPE.STRING, Signature);
                    /* 3 */
                    _pm.XMLTXParser_Set_End();                   
                }
            }
        }

		/**
		* This is a function used in order to initialize a verification operation,
		* where the signature is an appendix to the data
		*/
		private void CS_VerifyInit()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;		
			// Function arguments IN
			UInt32                              hSession;			
			CryptoCore.Smf.MECHANISM            mech;
			UInt32								hKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey}	   // 
			/////////////////////////////////////////////////////////////////
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
			/* get mech */
			ret = converter.ParseMechanismIn(out mech, config);			
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - VerifyInit
				ret = _smf.fvsm.VerifyInit(mech, hKey);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);					
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_VerifyInit", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}

		/**
		* This is a function used in order to verify a signature in a single-part operation,
		* where the signature is an appendix to the data
		*/
		private void CS_Verify()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              Data;			
			string                              Signature;


			//////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                           //
			// Param: {UInt32 hSession, string Data, string Signature}				// 
			//////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get Data */
			Data = ((ProtocolManager.Param)config.body.param["Data"]).vvalue;
			/* get Signature */
			Signature = ((ProtocolManager.Param)config.body.param["Signature"]).vvalue;
			
			// Send Command - Verify
			ret = _smf.fvsm.Verify(Data, Signature);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_Verify", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

        /**
		* This is a function used in order to initialize a verification operation and verify a signature in a single-part operation,
		* where the signature is an appendix to the data
		*/
        private void CS_CompactVerify()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            CryptoCore.CCConvert converter;
            // Function arguments IN
            UInt32 hSession;
            CryptoCore.Smf.MECHANISM mech;
            UInt32 hKey;
            string Data;
            string Signature;


            // Initialization
            converter = new CryptoCore.CCConvert();

            /////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                  //
            // Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey,       //
            // string Data, string Signature}	                           // 
            /////////////////////////////////////////////////////////////////
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;
            /* get Data */
            Data = ((ProtocolManager.Param)config.body.param["Data"]).vvalue;
            /* get Signature */
            Signature = ((ProtocolManager.Param)config.body.param["Signature"]).vvalue;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
            /* get mech */
            ret = converter.ParseMechanismIn(out mech, config);            

            // Bad Result
            if (ret != ErrorManager.DKMS_OK)
            {
                _pm.XMLTXParser_Set_Error(ret);
            }
            // Good Result
            else
            {
                // Send Command - CompactVerify
                ret = _smf.fvsm.CompactVerify(mech, hKey, Data, Signature);

                // Bad Result
                if (ret != ErrorManager.DKMS_OK)
                {
                    _pm.XMLTXParser_Set_Error(ret);
                }
                // Good Result
                else
                {                    
                    //////////////////////////////////////
                    // Fase 3 Build the responce
                    /* 1 */
                    _pm.XMLTXParser_Set_Start("DKMS_CompactVerify", 1);
                    /* 2 */
                    _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                    /* 3 */
                    _pm.XMLTXParser_Set_End();                   
                }
            }
        }

		/**
		* This is a function used in order to continue a multiple-part verification operation,
		* processing another data part		
		*/
		private void CS_VerifyUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              PartData;			


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                                   //
			// Param: {UInt32 hSession, string PartData}									// 
			//////////////////////////////////////////////////////////////////////////////////
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get PartData */
			PartData = ((ProtocolManager.Param)config.body.param["PartData"]).vvalue;
			
			// Send Command - VerifyUpdate
			ret = _smf.fvsm.VerifyUpdate(PartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_VerifyUpdate", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to finish a multiple-part verification operation,
		* checking the signature
		*/
		private void CS_VerifyFinal()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;			
			string                              Signature;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, string Signature}				   // 
			/////////////////////////////////////////////////////////////////		
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get Signature */
			Signature = ((ProtocolManager.Param)config.body.param["Signature"]).vvalue;
			
			// Send Command - VerifyFinal
			ret = _smf.fvsm.VerifyFinal(Signature);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_VerifyFinal", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}
		}

		/**
		* This is a function used in order to initialize a signature verification operation,
		* where the data is recovered from the signature
		*/
		private void CS_VerifyRecoverInit()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;		
			// Function arguments IN
			UInt32                              hSession;			
			CryptoCore.Smf.MECHANISM            mech;
			UInt32								hKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey}	   // 
			/////////////////////////////////////////////////////////////////
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
			/* get mech */
			ret = converter.ParseMechanismIn(out mech, config);			
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - VerifyRecoverInit
				ret = _smf.fvsm.VerifyRecoverInit(mech, hKey);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);					
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_VerifyRecoverInit", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}

		/**
		* This is a function used in order to verify a signature in a single-part operation,
		* where the data is recovered from the signature
		*/
		private void CS_VerifyRecover()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			string                              Signature;
			// Function arguments OUT
			string								Data;			


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, out string Data, string Signature} // 
			/////////////////////////////////////////////////////////////////		
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);			
			_smf.CurrentSession = hSession;
			/* get Signature */
			Signature = ((ProtocolManager.Param)config.body.param["Signature"]).vvalue;
			
			// Send Command - VerifyRecover
			ret = _smf.fvsm.VerifyRecover(out Data, Signature);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_VerifyRecover", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "Data", ProtocolManager.PARAM_TYPE.STRING, Data);
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}
		}

        /**
		* This is a function used in order to initialize a signature verification operation and verify a signature in a single-part operation,
		* where the data is recovered from the signature
		*/
        private void CS_CompactVerifyRecover()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            CryptoCore.CCConvert                converter;
            // Function arguments IN
            UInt32                              hSession;
            CryptoCore.Smf.MECHANISM            mech;
            UInt32                              hKey;
            string                              Signature;
            // Function arguments OUT
            string                              Data;


            // Initialization
            converter = new CryptoCore.CCConvert();

            /////////////////////////////////////////////////////////////////				
            // Get parameter from DKMSProtocol Data Block                  //
            // Param: {UInt32 hSession, MECHANISM mech, UInt32 hKey,       //
            // out string Data, string Signature}	                       // 
            /////////////////////////////////////////////////////////////////
            hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
            _smf.CurrentSession = hSession;
            /* get Signature */
            Signature = ((ProtocolManager.Param)config.body.param["Signature"]).vvalue;
            /* get hKey */
            hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
            /* get mech */
            ret = converter.ParseMechanismIn(out mech, config);            

            // Bad Result
            if (ret != ErrorManager.DKMS_OK)
            {
                _pm.XMLTXParser_Set_Error(ret);
            }
            // Good Result
            else
            {
                // Send Command - CompactVerifyRecover
                ret = _smf.fvsm.CompactVerifyRecover(mech, hKey, out Data, Signature);

                // Bad Result
                if (ret != ErrorManager.DKMS_OK)
                {
                    _pm.XMLTXParser_Set_Error(ret);
                }
                // Good Result
                else
                {                    
                    //////////////////////////////////////
                    // Fase 3 Build the responce					
                    /* 1 */
                    _pm.XMLTXParser_Set_Start("DKMS_CompactVerifyRecover", 2);
                    /* 2 */
                    _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                    _pm.XMLTXParser_Set_ParamAtomic(1, "Data", ProtocolManager.PARAM_TYPE.STRING, Data);
                    /* 3 */
                    _pm.XMLTXParser_Set_End();                    
                }
            }
        }

		/**
		* This is a function used in order to continue multiple-part digest and encryption operations,
		* processing another data part	
		*/
		private void CS_DigestEncryptUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              ClearPartData;
			// Function arguments OUT				
			string                              EncryptedPartData;


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block									//
			// Param: {UInt32 hSession, string ClearPartData, out string EncryptedPartData} // 
			//////////////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get ClearPartData */
			ClearPartData = ((ProtocolManager.Param)config.body.param["ClearPartData"]).vvalue;
			
			// Send Command - DigestEncryptUpdate
			ret = _smf.dfcf.DigestEncryptUpdate(ClearPartData, out EncryptedPartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DigestEncryptUpdate", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "EncryptedPartData", ProtocolManager.PARAM_TYPE.STRING, EncryptedPartData);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to continue a multiple-part combined decryption and digest
		* operation, processing another data part	
		*/
		private void CS_DecryptDigestUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              EncryptedPartData;
			// Function arguments OUT				
			string                              ClearPartData;


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block									//
			// Param: {UInt32 hSession, string EncryptedPartData, out string ClearPartData} // 
			//////////////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get EncryptedPartData */
			EncryptedPartData = ((ProtocolManager.Param)config.body.param["EncryptedPartData"]).vvalue;
			
			// Send Command - DecryptDigestUpdate
			ret = _smf.dfcf.DecryptDigestUpdate(EncryptedPartData, out ClearPartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DecryptDigestUpdate", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "ClearPartData", ProtocolManager.PARAM_TYPE.STRING, ClearPartData);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to continue a multiple-part combined signature and encryption
		* operation, processing another data part	
		*/
		private void CS_SignEncryptUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              ClearPartData;
			// Function arguments OUT				
			string                              EncryptedPartData;


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block									//
			// Param: {UInt32 hSession, string ClearPartData, out string EncryptedPartData} // 
			//////////////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get ClearPartData */
			ClearPartData = ((ProtocolManager.Param)config.body.param["ClearPartData"]).vvalue;
			
			// Send Command - SignEncryptUpdate
			ret = _smf.dfcf.SignEncryptUpdate(ClearPartData, out EncryptedPartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_SignEncryptUpdate", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "EncryptedPartData", ProtocolManager.PARAM_TYPE.STRING, EncryptedPartData);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to continue a multiple-part combined decryption and verification
		* operation, processing another data part
		*/
		private void CS_DecryptVerifyUpdate()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;				
			string                              EncryptedPartData;
			// Function arguments OUT				
			string                              ClearPartData;


			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block									//
			// Param: {UInt32 hSession, string EncryptedPartData, out string ClearPartData} // 
			//////////////////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get EncryptedPartData */
			EncryptedPartData = ((ProtocolManager.Param)config.body.param["EncryptedPartData"]).vvalue;
			
			// Send Command - DecryptVerifyUpdate
			ret = _smf.dfcf.DecryptVerifyUpdate(EncryptedPartData, out ClearPartData);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_DecryptVerifyUpdate", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "ClearPartData", ProtocolManager.PARAM_TYPE.STRING, ClearPartData);
				/* 3 */
				_pm.XMLTXParser_Set_End();
			}
		}

		/**
		* This is a function used in order to generate a public/private key pair,
		* creating new key objects		
		*/
		private void CS_GenerateKeyPair()
		{                
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;
			UInt32								ulPublicKeyAttributeCount;
			UInt32								ulPrivateKeyAttributeCount;
			// Function arguments IN
			UInt32                              hSession;
			CryptoCore.ATTRIBUTE_VALUE          templpb;
			CryptoCore.ATTRIBUTE_VALUE          templpv;
			CryptoCore.Smf.MECHANISM            mech;
			// Function arguments OUT				
			UInt32                              phpbKey;
			UInt32                              phpvKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			///////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                                    //
			// Param: {UInt32 hSession, MECHANISM mech, CryptoCore.ATTRIBUTE_VALUE templpb,  //
			// CryptoCore.ATTRIBUTE_VALUE templpv,	out UInt32 phpbKey, out UInt32 phpvKey}  // 
			///////////////////////////////////////////////////////////////////////////////////		
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get templpb */ 
			ulPublicKeyAttributeCount = ((ProtocolManager.Param)config.body.param["templpb"]).XDim;
			ret = converter.ParseAttributeIn(config, "templpb", ulPublicKeyAttributeCount, out templpb);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				/* get templpv */ 
				ulPrivateKeyAttributeCount = ((ProtocolManager.Param)config.body.param["templpv"]).XDim;
				ret = converter.ParseAttributeIn(config, "templpv", ulPrivateKeyAttributeCount, out templpv);
				
				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);				
				}
				// Good Result
				else
				{								
					/* get mech */
					ret = converter.ParseMechanismIn(out mech, config);

					// Bad Result
					if (ret != ErrorManager.DKMS_OK)
					{
						_pm.XMLTXParser_Set_Error(ret);				
					}
					// Good Result
					else
					{
						// Send Command - GenerateKeyPair
						ret = _smf.kmf.GenerateKeyPair(mech, templpb, templpv, out phpbKey, out phpvKey);

						// Bad Result
						if (ret != ErrorManager.DKMS_OK)
						{
							_pm.XMLTXParser_Set_Error(ret);					
						}
						// Good Result
						else
						{
							//////////////////////////////////////
							// Fase 3 Build the responce					
							/* 1 */
							_pm.XMLTXParser_Set_Start("DKMS_GenerateKeyPair", 3);
							/* 2 */
							_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
							_pm.XMLTXParser_Set_ParamAtomic(1, "phpbKey", ProtocolManager.PARAM_TYPE.UINT32, phpbKey.ToString());
							_pm.XMLTXParser_Set_ParamAtomic(2, "phpvKey", ProtocolManager.PARAM_TYPE.UINT32, phpvKey.ToString());
							/* 3 */
							_pm.XMLTXParser_Set_End();
						}
					}
				}
			}
		}

		/**
		* This is a function used in order to wrap (encrypts) a private or secret key		
		*/
		private void CS_WrapKey()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;		
			// Function arguments IN
			UInt32                              hSession;
			UInt32                              hWrappingKey;	
			UInt32                              hKey;
			CryptoCore.Smf.MECHANISM            mech;
			// Function arguments OUT
			string                              WrappedKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			//////////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block									//
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hWrappingKey, UInt32 hKey,	//
			// out string WrappedKey}														// 
			//////////////////////////////////////////////////////////////////////////////////
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hWrappingKey */ 
			hWrappingKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hWrappingKey"]).vvalue);
			/* get hKey */ 
			hKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hKey"]).vvalue);
			/* get mech */
			ret = converter.ParseMechanismIn(out mech, config);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				// Send Command - WrapKey
				ret = _smf.kmf.WrapKey(mech, hWrappingKey, hKey, out WrappedKey);

				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);					
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce					
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_WrapKey", 2);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
					_pm.XMLTXParser_Set_ParamAtomic(1, "WrappedKey", ProtocolManager.PARAM_TYPE.STRING, WrappedKey);
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}

		/**
		* This is a function used in order to Unwraps (decrypts) a wrapped key,
		* creating a new private key or secret key object		
		*/
		private void CS_UnwrapKey()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;
			UInt32								ulCount;
			// Function arguments IN
			UInt32                              hSession;
			UInt32								hUnwrappingKey;
			string								WrappedKey;
			CryptoCore.ATTRIBUTE_VALUE          templ;			
			CryptoCore.Smf.MECHANISM            mech;
			// Function arguments OUT
			UInt32                              phKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			//////////////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block								//
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hUnwrappingKey,			// 
			// string WrappedKey, CryptoCore.ATTRIBUTE_VALUE templ, out UInt32 phKey}   // 
			//////////////////////////////////////////////////////////////////////////////
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hUnwrappingKey */ 
			hUnwrappingKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hUnwrappingKey"]).vvalue);
			/* get WrappedKey */ 
			WrappedKey = ((ProtocolManager.Param)config.body.param["WrappedKey"]).vvalue;
			/* get templ */	
			ulCount = ((ProtocolManager.Param)config.body.param["templ"]).XDim;			
			ret = converter.ParseAttributeIn(config, "templ", ulCount, out templ);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{		
				/* get mech */
				ret = converter.ParseMechanismIn(out mech, config);
			
				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);				
				}
				// Good Result
				else
				{			
					// Send Command - UnwrapKey
					ret = _smf.kmf.UnwrapKey(mech, hUnwrappingKey, WrappedKey, templ, out phKey);

					// Bad Result
					if(ret != ErrorManager.DKMS_OK)
					{
						_pm.XMLTXParser_Set_Error(ret);				
					}
					// Good Result
					else
					{
						//////////////////////////////////////
						// Fase 3 Build the responce					
						/* 1 */
						_pm.XMLTXParser_Set_Start("DKMS_UnwrapKey", 2);
						/* 2 */
						_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
						_pm.XMLTXParser_Set_ParamAtomic(1, "phKey", ProtocolManager.PARAM_TYPE.UINT32, phKey.ToString());
						/* 3 */
						_pm.XMLTXParser_Set_End();
					}
				}
			}
		}

		/**
		* This is a function used in order to derive a key from a base key, creating a new key object
		*/
		private void CS_DeriveKey()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;
			UInt32								ulCount;
			// Function arguments IN
			UInt32                              hSession;
			UInt32								hBaseKey;			
			CryptoCore.ATTRIBUTE_VALUE          templ;			
			CryptoCore.Smf.MECHANISM            mech;
			// Function arguments OUT
			UInt32                              phKey;


			// Initialization
			converter = new CryptoCore.CCConvert();

			//////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block						//
			// Param: {UInt32 hSession, MECHANISM mech, UInt32 hBaseKey,		//
			// CryptoCore.ATTRIBUTE_VALUE templ, out UInt32 phKey}				// 
			//////////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hBaseKey */ 
			hBaseKey = UInt32.Parse(((ProtocolManager.Param)config.body.param["hBaseKey"]).vvalue);			
			/* get templ */
			ulCount = ((ProtocolManager.Param)config.body.param["templ"]).XDim;			
			ret = converter.ParseAttributeIn(config, "templ", ulCount, out templ);
			
			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				/* get mech */
				ret = converter.ParseMechanismIn(out mech, config);
			
				// Bad Result
				if(ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);				
				}
				// Good Result
				else
				{			
					// Send Command - DeriveKey
					ret = _smf.kmf.DeriveKey(mech, hBaseKey, templ, out phKey);

					// Bad Result
					if(ret != ErrorManager.DKMS_OK)
					{
						_pm.XMLTXParser_Set_Error(ret);				
					}
					// Good Result
					else
					{
						//////////////////////////////////////
						// Fase 3 Build the responce					
						/* 1 */
						_pm.XMLTXParser_Set_Start("DKMS_DeriveKey", 2);
						/* 2 */
						_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
						_pm.XMLTXParser_Set_ParamAtomic(1, "phKey", ProtocolManager.PARAM_TYPE.UINT32, phKey.ToString());
						/* 3 */
						_pm.XMLTXParser_Set_End();
					}
				}
			}
		}

		/**
		* This is a function used in order to modifie the value of one or more attributes of an object	
		*/
		private void CS_SetAttributeValue()
		{                
			UInt32								ret = ErrorManager.DKMS_OK;
			CryptoCore.CCConvert				converter;
			UInt32								ulCount;
			// function arguments in
			UInt32                              hSession;
			UInt32                              hObject;
            CryptoCore.ATTRIBUTE_VALUE          templ;    
			
			
			// Initialization
			converter = new CryptoCore.CCConvert();

			/////////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                      //
			// Param: {UInt32 hSession, UInt32 hObject, ATTRIBUTE_VALUE templ} // 
			/////////////////////////////////////////////////////////////////////			
			/* get-set hSession */	
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hObject */
			hObject = UInt32.Parse(((ProtocolManager.Param)config.body.param["hObject"]).vvalue);
			/* get templ */	
			ulCount = ((ProtocolManager.Param)config.body.param["templ"]).XDim;
			ret = converter.ParseAttributeIn(config, "templ", ulCount, out templ);
			
			// Bad Result
			if (ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);			
			}
			// Good Result			
			else
			{
				// Send Command - SetAttributeValue
				ret = _smf.omf.SetAttributeValue(hObject, templ);

				// Bad Result
				if (ret != ErrorManager.DKMS_OK)
				{
					_pm.XMLTXParser_Set_Error(ret);			
				}
				// Good Result
				else
				{
					//////////////////////////////////////
					// Fase 3 Build the responce
					/* 1 */
					_pm.XMLTXParser_Set_Start("DKMS_SetAttributeValue", 1);
					/* 2 */
					_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
					/* 3 */
					_pm.XMLTXParser_Set_End();
				}
			}
		}

		/**
		* This is a function used in order to get the size of an object in bytes
		*/
		private void CS_GetObjectSize()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			UInt32								hObject;
			// Function arguments OUT
			UInt32								pulSize;


			//////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                   //
			// Param: {UInt32 hSession, UInt32 hObject, out UInt32 pulSize} // 
			//////////////////////////////////////////////////////////////////
			/* get hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get hObject */
			hObject = UInt32.Parse(((ProtocolManager.Param)config.body.param["hObject"]).vvalue);

			// Send Command - GetObjectSize			
			ret = _smf.omf.GetObjectSize(hObject, out pulSize);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetObjectSize", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "pulSize", ProtocolManager.PARAM_TYPE.UINT32, pulSize.ToString());
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}

		/**
		* This is a function used in order to modifie the PIN of the user that is currently logged in		
		*/
		private void CS_SetPIN()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			string								OldPin;
			string								NewPin;
			

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, string OldPin, string NewPin}      // 
			/////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get OldPin */
			OldPin = ((ProtocolManager.Param)config.body.param["OldPin"]).vvalue;
			/* get NewPin */
			NewPin = ((ProtocolManager.Param)config.body.param["NewPin"]).vvalue;
			
			// Send Command - SetPIN
			ret = _smf.SetPIN(OldPin, NewPin);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_SetPIN", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}

		/**
		* This is a function used in order to initialize the normal users PIN		
		*/
		private void CS_InitPIN()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              hSession;
			string								Pin;		
			

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 hSession, string Pin}						   // 
			/////////////////////////////////////////////////////////////////			
			/* get-set hSession */
			hSession = UInt32.Parse(((ProtocolManager.Param)config.body.param["hSession"]).vvalue);
			_smf.CurrentSession = hSession;
			/* get Pin */
			Pin = ((ProtocolManager.Param)config.body.param["Pin"]).vvalue;
						
			// Send Command - InitPIN
			ret = _smf.InitPIN(Pin);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_InitPIN", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}

		/**
		* This is a function used in order to initialize a token		
		*/
		private void CS_InitToken()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              slotID;
			string								Pin;
			string								Label;
			

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 slotID, string Pin, string Label}			   // 
			/////////////////////////////////////////////////////////////////			
			/* get slotID */
			slotID = UInt32.Parse(((ProtocolManager.Param)config.body.param["slotID"]).vvalue);			
			/* get Pin */
			Pin = ((ProtocolManager.Param)config.body.param["Pin"]).vvalue;
			/* get Label */
			Label = ((ProtocolManager.Param)config.body.param["Label"]).vvalue;
			Label = Label.Substring(0, Label.Length - 1);
						
			// Send Command - InitToken
			ret = _stmf.InitToken(slotID, Pin, Label);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_InitToken", 1);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());				
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}

		/**
		* This is a function used in order to obtains information about a particular mechanism		
		*/
		private void CS_GetMechanismInfo()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              slotID;
			CryptoCore.Smf.CK_MECHANISM_TYPE	type;
			// Function arguments OUT
			CryptoCore.Stmf.CK_MECHANISM_INFO	pInfo;
			

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 slotID, Smf.CK_MECHANISM_TYPE type,		   //	
			// out CK_MECHANISM_INFO pInfo}								   // 
			/////////////////////////////////////////////////////////////////			
			/* get slotID */
			slotID = UInt32.Parse(((ProtocolManager.Param)config.body.param["slotID"]).vvalue);			
			/* get type */
			type = (CryptoCore.Smf.CK_MECHANISM_TYPE)int.Parse(((ProtocolManager.Param)config.body.param["type"]).vvalue);
			
			////////////////////////////////////////////////
			// Initialization                             //
			////////////////////////////////////////////////			
			/* initialize pInfo */
			pInfo = new CryptoCore.Stmf.CK_MECHANISM_INFO();

			// Send Command - GetMechanismInfo
			ret = _stmf.GetMechanismInfo(slotID, type, out pInfo);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetMechanismInfo", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_StartParamStruct(1, "pInfo", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.flags", ProtocolManager.PARAM_TYPE.INT32, pInfo.flags.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(2, "pInfo.ulMaxKeySize", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulMaxKeySize.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(3, "pInfo.ulMinKeySize", ProtocolManager.PARAM_TYPE.UINT32, pInfo.ulMinKeySize.ToString());
				_pm.XMLTXParser_Set_EndParamStruct();
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}

		/**
		* This is a function used in order to obtain information about a particular slot in the system
		*/
		private void CS_GetSlotInfo()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			UInt32                              slotID;			
			// Function arguments OUT
			CryptoCore.Stmf.CK_SLOT_INFO		pInfo;
			

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {UInt32 slotID, out CK_SLOT_INFO pInfo}			   //				
			/////////////////////////////////////////////////////////////////			
			/* get slotID */
			slotID = UInt32.Parse(((ProtocolManager.Param)config.body.param["slotID"]).vvalue);						
			
			////////////////////////////////////////////////
			// Initialization                             //
			////////////////////////////////////////////////			
			/* initialize pInfo */
			pInfo = new CryptoCore.Stmf.CK_SLOT_INFO();

			// Send Command - GetSlotInfo
			ret = _stmf.GetSlotInfo(slotID, out pInfo);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetSlotInfo", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_StartParamStruct(1, "pInfo", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_StartParamStruct(0, "pInfo.firmwareVersion", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(0, "pInfo.firmwareVersion.major", ProtocolManager.PARAM_TYPE.UINT32, pInfo.firmwareVersion.major.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.firmwareVersion.minor", ProtocolManager.PARAM_TYPE.UINT32, pInfo.firmwareVersion.minor.ToString());
				_pm.XMLTXParser_Set_EndParamStruct();
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.flags", ProtocolManager.PARAM_TYPE.UINT32, pInfo.flags.ToString());
				_pm.XMLTXParser_Set_StartParamStruct(2, "pInfo.hardwareVersion", ProtocolManager.PARAM_TYPE.STRUCT);
				_pm.XMLTXParser_Set_ParamAtomic(0, "pInfo.hardwareVersion.major", ProtocolManager.PARAM_TYPE.UINT32, pInfo.hardwareVersion.major.ToString());
				_pm.XMLTXParser_Set_ParamAtomic(1, "pInfo.hardwareVersion.minor", ProtocolManager.PARAM_TYPE.UINT32, pInfo.hardwareVersion.minor.ToString());
				_pm.XMLTXParser_Set_EndParamStruct();				
				_pm.XMLTXParser_Set_ParamAtomic(3, "pInfo.manufacturerID", ProtocolManager.PARAM_TYPE.STRING, pInfo.manufacturerID);
				_pm.XMLTXParser_Set_ParamAtomic(4, "pInfo.slotDescription", ProtocolManager.PARAM_TYPE.STRING, pInfo.slotDescription);
				_pm.XMLTXParser_Set_EndParamStruct();
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}

		/**
		* This is a function used in order to obtain a list of slots in the system
		*/
		private void CS_GetSlotList()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			bool								tokenPresent;			
			// Function arguments OUT
			UInt32[]							SlotList;
			

			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {bool tokenPresent, out UInt32[] SlotList}		   //				
			/////////////////////////////////////////////////////////////////			
			/* get tokenPresent */
			if (((ProtocolManager.Param)config.body.param["tokenPresent"]).vvalue != true.ToString())
				tokenPresent = false;
			else
				tokenPresent = true;			

			// Send Command - GetSlotList
			ret = _stmf.GetSlotList(tokenPresent, out SlotList);

			// Bad Result
			if(ret != ErrorManager.DKMS_OK)
			{
				_pm.XMLTXParser_Set_Error(ret);				
			}
			// Good Result
			else
			{
				//////////////////////////////////////
				// Fase 3 Build the responce					
				/* 1 */
				_pm.XMLTXParser_Set_Start("DKMS_GetSlotList", 2);
				/* 2 */
				_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
				_pm.XMLTXParser_Set_StartParamArray(1, "SlotList", ProtocolManager.PARAM_TYPE.UINT32, (UInt32)SlotList.Length, 0);				
				for(int i = 0; i < SlotList.Length; i++)
					_pm.XMLTXParser_Set_ParamAtomic((UInt32)i, "SlotList[" + i.ToString() + "]", ProtocolManager.PARAM_TYPE.UINT32, SlotList[i].ToString());				
				_pm.XMLTXParser_Set_EndParamArray();
				/* 3 */
				_pm.XMLTXParser_Set_End();				
			}			
		}
		
		/**
		* This is a function used in order to test that the device service requested by the
		* client it's the same of the current service
		*/
		private void CS_TestDeviceComunication()
		{				
			UInt32								ret = ErrorManager.DKMS_OK;			
			// Function arguments IN
			string								deviceName;


			/////////////////////////////////////////////////////////////////				
			// Get parameter from DKMSProtocol Data Block                  //
			// Param: {string deviceName}                                  // 
			/////////////////////////////////////////////////////////////////
			/* get deviceName */ 
			deviceName = ((ProtocolManager.Param)config.body.param["deviceName"]).vvalue;
			
			//////////////////////////////////////
			// Fase 3 Build the responce
			if(deviceName != "CryptoService")
			{
				ret = ErrorManager.DKMS_DEVICE_NOT_FOUND;				
			}							
			/* 1 */
			_pm.XMLTXParser_Set_Start("DKMS_TestDeviceComunication", 1);
			/* 2 */
			_pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
			/* 3 */
			_pm.XMLTXParser_Set_End();
		}

        /**
		* This is a function used in order to set the device policy
		*/
        private void CS_SetPolicy()
        {
            UInt32                              ret = ErrorManager.DKMS_OK;
            // Function arguments IN
            UInt32                              count;

            lock (typeof(Dispatcher))
            {
                /////////////////////////////////////////////////////////////////				
                // Get parameter from DKMSProtocol Data Block                  //
                // Param: {string[] PMethods}                                  // 
                /////////////////////////////////////////////////////////////////
                /* get count */
                count = ((ProtocolManager.Param)config.body.param["PMethods"]).XDim;

                // load methods in the container
                string delimStr = "|";
                char[] delimiter = delimStr.ToCharArray();

                _c.adminsgc.SyncAGContainer.Clear();
                _c.controllersgc.SyncCGContainer.Clear();
                _c.custodiansgc.SyncCGContainer.Clear();
                _c.insertersgc.SyncIGContainer.Clear();
                _c.operatorsgc.SyncOGContainer.Clear();
                _c.swadminsgc.SyncSGContainer.Clear();
                _c.swdevelopersgc.SyncSGContainer.Clear();
                _c.techniciansgc.SyncTGContainer.Clear();
                _c.adminsgc.SyncAGContainer.Add("SetPolicy", string.Empty);
                _c.adminsgc.SyncAGContainer.Add("TestDeviceComunication", string.Empty);
                _c.swadminsgc.SyncSGContainer.Add("SetPolicy", string.Empty);
                _c.swadminsgc.SyncSGContainer.Add("TestDeviceComunication", string.Empty);
                _c.swadminsgc.SyncSGContainer.Add("CloseSession", string.Empty);

                try
                {
                    for (int i = 0; i < (int)count; i++)
                    {
                        string[] policies = ((ProtocolManager.Param)config.body.param["PMethods[" + i.ToString() + "]"]).vvalue.Split(delimiter);
                        switch (policies[1])
                        {
                            case "dkms_swadmins":
                                if (!_c.swadminsgc.SyncSGContainer.ContainsKey(policies[0]))
                                    _c.swadminsgc.SyncSGContainer.Add(policies[0], string.Empty);
                                break;
                            case "dkms_admins":
                                if (!_c.adminsgc.SyncAGContainer.ContainsKey(policies[0]))
                                    _c.adminsgc.SyncAGContainer.Add(policies[0], string.Empty);
                                break;
                            case "dkms_controllers":
                                if (!_c.controllersgc.SyncCGContainer.ContainsKey(policies[0]))
                                    _c.controllersgc.SyncCGContainer.Add(policies[0], string.Empty);
                                break;
                            case "dkms_custodians":
                                if (!_c.custodiansgc.SyncCGContainer.ContainsKey(policies[0]))
                                    _c.custodiansgc.SyncCGContainer.Add(policies[0], string.Empty);
                                break;
                            case "dkms_inserters":
                                if (!_c.insertersgc.SyncIGContainer.ContainsKey(policies[0]))
                                    _c.insertersgc.SyncIGContainer.Add(policies[0], string.Empty);
                                break;
                            case "dkms_swdevelopers":
                                if (!_c.swdevelopersgc.SyncSGContainer.ContainsKey(policies[0]))
                                    _c.swdevelopersgc.SyncSGContainer.Add(policies[0], string.Empty);
                                break;
                            case "dkms_technicians":
                                if (!_c.techniciansgc.SyncTGContainer.ContainsKey(policies[0]))
                                    _c.techniciansgc.SyncTGContainer.Add(policies[0], string.Empty);
                                break;
                            case "dkms_operators":
                                if (!_c.operatorsgc.SyncOGContainer.ContainsKey(policies[0]))
                                    _c.operatorsgc.SyncOGContainer.Add(policies[0], string.Empty);
                                break;
                        }
                    }
                }
                catch
                {
                    ret = ErrorManager.DKMS_GENERAL_ERROR;
                }
                finally
                {
                    if (ret != ErrorManager.DKMS_OK)
                    {
                        _c.adminsgc.SyncAGContainer.Clear();
                        _c.controllersgc.SyncCGContainer.Clear();
                        _c.custodiansgc.SyncCGContainer.Clear();
                        _c.insertersgc.SyncIGContainer.Clear();
                        _c.operatorsgc.SyncOGContainer.Clear();
                        _c.swadminsgc.SyncSGContainer.Clear();
                        _c.swdevelopersgc.SyncSGContainer.Clear();
                        _c.techniciansgc.SyncTGContainer.Clear();
                        _c.adminsgc.SyncAGContainer.Add("SetPolicy", string.Empty);
                        _c.adminsgc.SyncAGContainer.Add("TestDeviceComunication", string.Empty);
                        _c.swadminsgc.SyncSGContainer.Add("SetPolicy", string.Empty);
                        _c.swadminsgc.SyncSGContainer.Add("TestDeviceComunication", string.Empty);
                        _c.swadminsgc.SyncSGContainer.Add("CloseSession", string.Empty);
                    }
                }

                //////////////////////////////////////
                // Fase 3 Build the responce                
                /* 1 */
                _pm.XMLTXParser_Set_Start("DKMS_SetPolicy", 1);
                /* 2 */
                _pm.XMLTXParser_Set_ParamAtomic(0, "ret", ProtocolManager.PARAM_TYPE.UINT32, ret.ToString());
                /* 3 */
                _pm.XMLTXParser_Set_End();
            }
        }

		#endregion

		#region Dispatcher class fields							
		
		/** The Stmf class instance */
		private CryptoCore.Stmf			_stmf;

		/** The DKMSProtocol class instance */
		private CryptoCore.Smf			_smf;

        /** Local copy of Containers class instance */
        private Containers              _c;        

		/** Local copy of CryptoCore class instance */    
        private CryptoCore				_cc;

		/** The DKMSProtocolManager class instance */
		private ProtocolManager			_pm;

		/** The local current Config structure field */
		private ProtocolManager.Config	config;

		#endregion
	}	
}
